<?php

MgInitializeWebTier(dirname(__FILE__)."/../webconfig.ini");
include ("../mapviewerphp/constants.php");

//Functions ripped from getselectedfeatures.php
function printDateTime($mgDateTime)
{
   $dayToday = $mgDateTime->GetDay();
   $month = $mgDateTime->GetMonth();
   $year = $mgDateTime->GetYear();
   return $dayToday.".".$month.".".$year;
}

function GetTypeName($propDef)
{
    if ($propDef->GetPropertyType() == MgFeaturePropertyType::DataProperty) {
        switch($propDef->GetDataType()) 
        {
            case MgPropertyType::Blob:
                return "Blob";
            case MgPropertyType::Boolean:
                return "Boolean";
            case MgPropertyType::Clob:
                return "Clob";
            case MgPropertyType::DateTime:
                return "DateTime";
            case MgPropertyType::Decimal:
                return "Decimal";
            case MgPropertyType::Double:
                return "Double";
            case MgPropertyType::Int16:
                return "Int16";
            case MgPropertyType::Int32:
                return "Int32";
            case MgPropertyType::Int64:
                return "Int64";
            case MgPropertyType::Single:
                return "Single";
            case MgPropertyType::String:
                return "String";
        }
    } else if ($propDef->GetPropertyType() == MgFeaturePropertyType::GeometricProperty) {
        return "Geometry";
    } else if ($propDef->GetPropertyType() == MgFeaturePropertyType::ObjectProperty) {
        return "Object";
    } else if ($propDef->GetPropertyType() == MgFeaturePropertyType::AssociationProperty) {
        return "Association";
    } else if ($propDef->GetPropertyType() == MgFeaturePropertyType::RasterProperty) {
        return "Raster";
    }
    
    return "Unknown";
}

function GetPropertyValueFromReader($fr, $agfRW, $wktRW, $propName)
{
    $val = "";
    $tStr = "";
    $propType = $fr->GetPropertyType($propName);
    switch($propType)
    {
        case MgPropertyType::Null:
            $tStr = "Null";
            $val = "";
            break;
        case MgPropertyType::Boolean:
            $tStr = "Boolean";
            $val = sprintf("%s", $fr->GetBoolean($propName));
            break;
        case MgPropertyType::Byte:
            $tStr = "Byte";
            $val = sprintf("%d", $fr->GetByte($propName));
            break;
        case MgPropertyType::DateTime:
            $tStr = "DateTime";
            $val = printDateTime($fr->GetDateTime($propName)); // yyyy-mm-dd is enforced regardless of locale
            break;
        case MgPropertyType::Single:
            $tStr = "Single";
            $val = sprintf("%f", $fr->GetSingle($propName));
            break;
        case MgPropertyType::Double:
            $tStr = "Double";
            $val = sprintf("%f", $fr->GetDouble($propName));
            break;
        case MgPropertyType::Int16:
            $tStr = "Int16";
            $val = sprintf("%d", $fr->GetInt16($propName));
            break;
        case MgPropertyType::Int32:
            $tStr = "Int32";
            $val = sprintf("%d", $fr->GetInt32($propName));
            break;
        case MgPropertyType::Int64:
            $tStr = "Int64";
            $val = sprintf("%d", $fr->GetInt64($propName));
            break;
        case MgPropertyType::String:
            $tStr = "String";
            $val = $fr->GetString($propName);
            break;
        case MgPropertyType::Blob: //NOT PRESENTABLE IN PROPERTY GRID
            $tStr = "BLOB";
            break;
        case MgPropertyType::Clob: //NOT PRESENTABLE IN PROPERTY GRID
            $tStr = "CLOB";
            break;
        case MgPropertyType::Feature: //NOT PRESENTABLE IN PROPERTY GRID
            $tStr = "Feature";
            break;
        case MgPropertyType::Geometry: //NOT PRESENTABLE IN PROPERTY GRID
            $tStr = "Geometry";
            $agf = $fr->GetGeometry($propName);
            $geom = $agfRW->Read($agf);
            $val = $wktRW->Write($geom);
            break;
        case MgPropertyType::Raster: //NOT PRESENTABLE IN PROPERTY GRID
            $tStr = "Raster";
            break;
    }
    return $val;
}

$fsId = new MgResourceIdentifier("Library://Samples/Sheboygan/Data/Parcels.FeatureSource");
$className = "Parcels";

$q1 = new MgFeatureAggregateOptions();
$q1->AddComputedProperty("RESULT", "UNIQUE(RTYPE)");

$q2 = new MgFeatureAggregateOptions();
$q2->AddComputedProperty("RESULT", "EQUAL_DIST(RLANDVC, 8, 500, 1000000)");

$q3 = new MgFeatureAggregateOptions();
$q3->AddComputedProperty("RESULT", "STDEV_DIST(RLANDVC, 8, 500, 1000000)");

$q4 = new MgFeatureAggregateOptions();
$q4->AddComputedProperty("RESULT", "QUANT_DIST(RLANDVC, 8, 500, 1000000)");

$q5 = new MgFeatureAggregateOptions();
$q5->AddComputedProperty("RESULT", "JENK_DIST(RLANDVC, 8, 500, 1000000)");

$q6 = new MgFeatureAggregateOptions();
$q6->AddComputedProperty("RESULT", "MINIMUM(RLANDVC)");

$q7 = new MgFeatureAggregateOptions();
$q7->AddComputedProperty("RESULT", "MAXIMUM(RLANDVC)");

$q8 = new MgFeatureAggregateOptions();
$q8->AddComputedProperty("RESULT", "MEAN(RLANDVC)");

$q9 = new MgFeatureAggregateOptions();
$q9->AddComputedProperty("RESULT", "STD_DEV(RLANDVC)");

$q10 = new MgFeatureAggregateOptions();
$q10->AddComputedProperty("RESULT", "EXTENT(SHPGEOM)");

$agfRW = new MgAgfReaderWriter();
$wktRW = new MgWktReaderWriter();

$siteConn = new MgSiteConnection();
$userInfo = new MgUserInformation("Anonymous", "");
$siteConn->Open($userInfo);
$featSvc = $siteConn->CreateService(MgServiceType::FeatureService);

$clsDef = $featSvc->GetClassDefinition($fsId, "SHP_Schema", $className);

$r1 = $featSvc->SelectAggregate($fsId, $className, $q1);
$r2 = $featSvc->SelectAggregate($fsId, $className, $q2);
$r3 = $featSvc->SelectAggregate($fsId, $className, $q3);
$r4 = $featSvc->SelectAggregate($fsId, $className, $q4);
$r5 = $featSvc->SelectAggregate($fsId, $className, $q5);
$r6 = $featSvc->SelectAggregate($fsId, $className, $q6);
$r7 = $featSvc->SelectAggregate($fsId, $className, $q7);
$r8 = $featSvc->SelectAggregate($fsId, $className, $q8);
$r9 = $featSvc->SelectAggregate($fsId, $className, $q9);
$r10 = $featSvc->SelectAggregate($fsId, $className, $q10);

?>
<html>
    <head>
        <title>MapGuide custom aggregate function examples</title>
    </head>
    <body>
    <h1>MapGuide custom aggregate function examples</h1>
    <div>
        <p>Feature Source: <?= $fsId->ToString() ?></p>
        <p>Feature Class: <?= $className ?></p>
        <p>Properties:</p>
        <ul>
        <?php
        $clsProps = $clsDef->GetProperties();
        for ($i = 0; $i < $clsProps->GetCount(); $i++)
        {
            $propDef = $clsProps->GetItem($i);
            echo "<li>".$propDef->GetName()." (".GetTypeName($propDef).")</li>";
        }
        ?>
        </ul>
    </div>
    <div>
        <p>Each example is executed like so: </p>
        <pre>
        
        $fsId = new MgResourceIdentifier("<?= $fsId->ToString() ?>");
        $className = "<?= $className ?>";
        $expr = "UNIQUE(Name)"; //or any other example here
        $query = new MgFeatureAggregateOptions();
        $query->AddComputedProperty("RESULT", $expr);
        
        $rdr = $featSvc->SelectAggregate($fsId, $className, $query);
        while ($rdr->ReadNext()) 
        {
            //Output the result
        }
        
        </pre>
    </div>
    <div>
        <h4>Example: UNIQUE(Name)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r1->ReadNext()) {
                $value = GetPropertyValueFromReader($r1, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: EQUAL_DIST(RLANDVC, 8, 500, 1000000)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r2->ReadNext()) {
                $value = GetPropertyValueFromReader($r2, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: STDEV_DIST(RLANDVC, 8, 500, 1000000)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r3->ReadNext()) {
                $value = GetPropertyValueFromReader($r3, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: QUANT_DIST(RLANDVC, 8, 500, 1000000)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r4->ReadNext()) {
                $value = GetPropertyValueFromReader($r4, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: JENK_DIST(RLANDVC, 8, 500, 1000000)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r5->ReadNext()) {
                $value = GetPropertyValueFromReader($r5, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: MINIMUM(RLANDVC)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r6->ReadNext()) {
                $value = GetPropertyValueFromReader($r6, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: MAXIMUM(RLANDVC)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r7->ReadNext()) {
                $value = GetPropertyValueFromReader($r7, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: MEAN(RLANDVC)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r8->ReadNext()) {
                $value = GetPropertyValueFromReader($r8, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: STD_DEV(RLANDVC)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r9->ReadNext()) {
                $value = GetPropertyValueFromReader($r9, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    
    <div>
        <h4>Example: EXTENT(SHPGEOM)</h4>
        <table border="1">
            <th>
                <tr>
                    <td><strong>RESULT</strong></td>
                </tr>
            </th>
            <?php
            while ($r10->ReadNext()) {
                $value = GetPropertyValueFromReader($r10, $agfRW, $wktRW, "RESULT");
            ?>
            <tr>
                <td><?= $value ?></td>
            </tr>
            <?php 
            }
            ?>
        </table>
    </div>
    </body>
</html>

<?php

$r1->Close();
$r2->Close();
$r3->Close();
$r4->Close();
$r5->Close();
$r6->Close();
$r7->Close();
$r8->Close();
$r9->Close();
$r10->Close();

?>